// 完毕：
using System;
using System.IO;
using System.Drawing;
using GrapeCity.Documents.Pdf;
using GrapeCity.Documents.Pdf.AcroForms;
using GrapeCity.Documents.Text;
using GrapeCity.Documents.Drawing;
using System.Security.Cryptography.X509Certificates;
using GCTEXT = GrapeCity.Documents.Text;
using GCDRAW = GrapeCity.Documents.Drawing;

namespace DsPdfWeb.Demos
{
    // 此示例演示了如何使用 .pfx 文件创建 PDF 并对其进行签名，
    // 使用 SignatureField 和签名图像。
    // 然后，该示例将签名的文件加载回另一个 GcPdfDocument 实例
    // 并验证签名。
    // 这个示例与 SignDoc 相同，但添加了代表签名的图像。
    public class VisualSignature
    {
        public int CreatePDF(Stream stream)
        {
            var doc = new GcPdfDocument();
            var page = doc.NewPage();
            var tf = new TextFormat() { Font = StandardFonts.Times, FontSize = 14 };
            page.Graphics.DrawString("你好世界！" +
                "下面由 DsPdfWeb VisualSignature 示例签名。",
                tf, new PointF(72, 72));

            // 初始化测试证书：
            var pfxPath = Path.Combine("Resources", "Misc", "DsPdfTest.pfx");
            var cert = new X509Certificate2(File.ReadAllBytes(pfxPath), "qq",
                X509KeyStorageFlags.MachineKeySet | X509KeyStorageFlags.PersistKeySet | X509KeyStorageFlags.Exportable);
            var sp = new SignatureProperties()
            {
                SignatureBuilder = new Pkcs7SignatureBuilder()
                {
                    CertificateChain = new X509Certificate2[] { cert }
                },
                Location = "DsPdfWeb Demo Browser",
                SignerName = "DsPdfWeb",
            };

            // 添加代表签名的图像：
            sp.SignatureAppearance.Image = GCDRAW.Image.FromFile(Path.Combine("Resources", "ImagesBis", "signature.png"));
            sp.SignatureAppearance.CaptionImageRelation = GrapeCity.Documents.Pdf.Annotations.CaptionImageRelation.ImageOnly;

            // 初始化一个签名字段来保存签名：
            var sf = new SignatureField();
            sf.Widget.Rect = new RectangleF(72, 72 * 2, 72 * 4, 36);
            sf.Widget.Page = page;
            sf.Widget.BackColor = Color.LightSeaGreen;
            sf.Widget.DefaultAppearance.Font = StandardFonts.Helvetica;
            sf.Widget.ButtonAppearance.Caption = $"Signer: {sp.SignerName}\r\nLocation: {sp.Location}";
            // 将签名字段添加到文档中：
            doc.AcroForm.Fields.Add(sf);
            // 连接签名字段和签名属性：
            sp.SignatureField = sf;

            // 签署并保存文档：
            // 笔记：
            // - 签名和保存是一个原子操作，两者不能分开。
            // - 传递给 Sign() 方法的流必须可读。
            doc.Sign(sp, stream);

            // 倒回流以读取刚刚创建的文档
            // 进入另一个 GcPdfDocument 并验证签名：
            stream.Seek(0, SeekOrigin.Begin);
            var doc2 = new GcPdfDocument();
            doc2.Load(stream);
            SignatureField sf2 = (SignatureField)doc2.AcroForm.Fields[0];
            if (!sf2.Value.VerifySignatureValue())
                throw new Exception("Failed to verify the signature");

            // 完成（生成并签名的文档已保存到“流”）。
            return doc.Pages.Count;
        }
    }
}
